import greenfoot.*;

/**
 * This world class can be used to secure all or part of a scenario
 * Three arguments can be passed to the constructor when a Pass object is created.
 * <l><li>The password that must be entered correctly to proceed</li>
 * <li>The world to proceed to when the password is entered correctly</li>
 * <li>The world to fall back to when the password is not entered correctly</li></l><br>
 * If 'null' is passed for the world to fall back to and the password is not entered
 * correctly after three attempts, then the scenario will stop. 
 * If 'null' is passed for the world to proceed to and the password is entered correctly,
 * then the scenario will proceed to the world given to fall back on.
 * The Pass world will not activate if no valid world is passed.
 * The Pass world will activate automatically when created if a valid world is passed.
 * (in other words, the code to set this world as the active world is contained in the
 * class constructor).
 */
public class Pass extends World
{
    private static int maxWordLen = 8;
    private World passWorld, failWorld;
    private String description = "", answer, input = "";
    private int attempts = 3;
    
    /**
     * This constructor can be used to prevent any further action unless the password is entered correctly
     *
     * @param correctInput: the password that must be entered correctly
     * @param nextWorld: the world to proceed to when password is entered correctly
     */
    public Pass(String correctInput, World nextWorld)
    {
        this(correctInput, "", nextWorld, null);
    }
    
    /**
     * This constructor can be used to prevent any further action unless the password is entered correctly and will
     * display a custom text message.
     *
     * @param correctInput: the password that must be entered correctly
     * @param descriptor: a text message that is displayed on the security screen
     * @param nextWorld: the world to proceed to when password is entered correctly
     */
    public Pass(String correctInput, String descriptor, World nextWorld)
    {
        this(correctInput, descriptor, nextWorld, null);
    }
    
    /**
     * This constructor can be used to prevent any further action; or,
     * it can be used to only proceed to one world when the password entered is correct, else it proceeds
     * to another world.
     *
     * @param correctInput A parameter
     * @param nextWorld A parameter
     * @param prevWorld A parameter
     */
    public Pass(String correctInput, World nextWorld, World prevWorld)
    {
        this(correctInput, "", nextWorld, prevWorld);
    }
    
    /**
     * This constructor can be used to prevent any further action; or,
     * it can be used to only proceed to one world when the password entered is correct, else it proceeds
     * to another world.  It will also display a custom text message.
     *
     * @param correctInput A parameter
     * @param descriptor: a text message that is displayed on the security screen
     * @param prevWorld A parameter
     */
    public Pass(String correctInput, String descriptor, World nextWorld, World prevWorld)
    {
        super(300, 400, 1);
        answer = correctInput;
        if (answer.length() > maxWordLen) answer = answer.substring(0, maxWordLen);
        description = descriptor;
        passWorld = nextWorld;
        failWorld = prevWorld;
        updateBackground();
        if (passWorld == null && failWorld == null) return;
        Greenfoot.setWorld(this);        
    }
    
    /**
     * This method allows user three more attemps after restarting the scenario.<br>
     * <br>
     * NOTE: this method is called by the system when the scenario is re-started.
     */
    public void started()
    {
        attempts = 3;
    }
    
    private void updateBackground()
    {
        GreenfootImage bg = getBackground();
        bg.setColor(Color.BLUE);
        bg.fill();
        bg.setColor(Color.LIGHT_GRAY);
        bg.fillRect(10, 10, getWidth()-20, getHeight()-20);
        bg.setColor(Color.BLACK);
        GreenfootImage text = new GreenfootImage("Security", 60, Color.BLACK, new Color(0, 0, 0, 0));
        bg.drawImage(text, (getWidth()-text.getWidth())/2, 40);
        text = new GreenfootImage(description, 24, new Color(128, 0, 0), new Color(0, 0, 0, 0));
        bg.drawImage(text, (getWidth()-text.getWidth())/2, 100);
        text = new GreenfootImage("Enter password", 36, Color.BLACK, new Color(0, 0, 0, 0));
        bg.drawImage(text, (getWidth()-text.getWidth())/2, 200);
        text = new GreenfootImage("*", 36, Color.BLACK, new Color(0, 0, 0, 0));
        for (int i=0; i<input.length(); i++) bg.drawImage(text, getWidth()/2-maxWordLen*10+i*20, 280);
    }
    
    /**
     * Actions involve detecting and acting upon keystrokes.
     * The following keystrokes are allowed:
     * <l><li>'enter' tests entered characters to the password</li>
     * <li>'escape' removes all characters entered</li>
     * <li>'backspace' removes the last character entered</li>
     * <li>'space' and any standard character adds that character to those entered</li></l><br>
     * The number of characters that can be entered is limited to 'maxWordLen'.
     */
    public void act()
    {
        String key = Greenfoot.getKey();
        if (key == null) return;
        boolean update = false;
        if ("enter".equals(key))
        {
            if (!answer.equals(input))
            {
                GreenfootImage text = new GreenfootImage("Access Denied", 36, Color.RED, new Color(0, 0, 0, 0));
                getBackground().drawImage(text, (getWidth()-text.getWidth())/2, 350);
                Greenfoot.delay(100);
                attempts--;
                if (attempts == 0)
                {
                    if (failWorld == null) Greenfoot.stop(); else Greenfoot.setWorld(failWorld);
                    return;
                }
                input = "";
                update = true;
            }
            else
            {
                GreenfootImage text = new GreenfootImage("Access Granted", 36, new Color(0, 128, 0), new Color(0, 0, 0, 0));
                getBackground().drawImage(text, (getWidth()-text.getWidth())/2, 350);
                Greenfoot.delay(100);
                if (passWorld != null) Greenfoot.setWorld(passWorld); else Greenfoot.setWorld(failWorld);
                return;
            }
        }
        if ("escape".equals(key))
        {
            if ("".equals(input) && failWorld != null) Greenfoot.setWorld(failWorld);
            input = "";
            update = true;
        }
        if ("backspace".equals(key) && input.length() > 0)
        {
            input = input.substring(0, input.length()-1);
            update = true;
        }
        if ("space".equals(key)) key = " ";
        if (key.length() == 1 && input.length() < maxWordLen)
        {
            input += key;
            update = true;
        }
        if (update) updateBackground();
    }
    
    /**
     * This class method will set the maximum accepted password length to the value given.
     *
     * @param len: the value to set the maximum password length to
     */
    public static void setMaximumLength(int len)
    {
        maxWordLen = len;
    }
}